package org.xpilot.jxpmap;

import java.awt.event.MouseEvent;

public class CanvasEventAdapter implements CanvasEventHandler {
        public void mouseClicked (MouseEvent evt) {}
        public void mouseEntered (MouseEvent evt) {}
        public void mouseExited (MouseEvent evt) {}
        public void mousePressed (MouseEvent evt) {}
        public void mouseReleased (MouseEvent evt) {}
        public void mouseDragged (MouseEvent evt) {}
        public void mouseMoved (MouseEvent evt) {}
}
