//Microsoft Developer Studio generated resource script.
//
#include "resource.h"

#define APSTUDIO_READONLY_SYMBOLS
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//
#include "afxres.h"

/////////////////////////////////////////////////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
// English (U.S.) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENU)
#ifdef _WIN32
LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_US
#pragma code_page(1252)
#endif //_WIN32

#ifdef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE DISCARDABLE 
BEGIN
    "resource.h\0"
END

2 TEXTINCLUDE DISCARDABLE 
BEGIN
    "#include ""afxres.h""\r\n"
    "\0"
END

3 TEXTINCLUDE DISCARDABLE 
BEGIN
    "#define _AFX_NO_SPLITTER_RESOURCES\r\n"
    "#define _AFX_NO_OLE_RESOURCES\r\n"
    "#define _AFX_NO_TRACKER_RESOURCES\r\n"
    "#define _AFX_NO_PROPERTY_RESOURCES\r\n"
    "\r\n"
    "#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENU)\r\n"
    "#ifdef _WIN32\r\n"
    "LANGUAGE 9, 1\r\n"
    "#pragma code_page(1252)\r\n"
    "#endif\r\n"
    "#include ""res\\xpilot.rc2""  // non-Microsoft Visual C++ edited resources\r\n"
    "#include ""afxres.rc""         // Standard components\r\n"
    "#endif\0"
END

#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.
IDR_MAINFRAME           ICON    DISCARDABLE     "res\\xpilot.ico"
IDR_XPILOTTYPE          ICON    DISCARDABLE     "res\\xpilot.ico"
//IDI_XPILOTS             ICON    DISCARDABLE     "..\\..\\server\\NT\\res\\xpilots.ico"
IDI_ICON1               ICON    DISCARDABLE     "res\\xpilot.ico"

/////////////////////////////////////////////////////////////////////////////
//
// Bitmap
//

IDB_SPLASH              BITMAP  DISCARDABLE     "res\\Splsh16.bmp"

/////////////////////////////////////////////////////////////////////////////
//
// Toolbar
//

IDR_MAINFRAME TOOLBAR DISCARDABLE  16, 15
BEGIN
    BUTTON      ID_FILE_NEW
    BUTTON      ID_FILE_OPEN
    BUTTON      ID_FILE_SAVE
    SEPARATOR
    BUTTON      ID_EDIT_CUT
    BUTTON      ID_EDIT_COPY
    BUTTON      ID_EDIT_PASTE
    SEPARATOR
    BUTTON      ID_FILE_PRINT
    BUTTON      ID_APP_ABOUT
    BUTTON      ID_CONTEXT_HELP
END


/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

IDD_ABOUTBOX DIALOG DISCARDABLE  0, 0, 339, 241
STYLE DS_MODALFRAME | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "About XPilot for Windows"
FONT 8, "MS Sans Serif"
BEGIN
    ICON            IDR_MAINFRAME,IDC_STATIC,7,17,20,20
    LTEXT           "Version 0.8",IDC_VERSION,41,23,252,8,SS_NOPREFIX
    CTEXT           "XPilot is Copyright © 1991-2002",IDC_STATIC,7,41,325,8
    DEFPUSHBUTTON   "OK",IDOK,153,219,32,14,WS_GROUP
    CTEXT           "by Bjørn Stabell, Ken Ronny Schouten, Bert Gijsbers && Dick Balaska.",
                    IDC_STATIC,7,52,325,8
    ICON            IDI_XPILOTS,IDC_STATIC,314,17,20,20
    CONTROL         "",IDC_CREDITS,"Static",SS_BLACKFRAME,7,71,325,137
END

IDD_MOTD DIALOG DISCARDABLE  0, 0, 258, 190
STYLE DS_MODALFRAME | DS_CENTER | WS_POPUP | WS_VISIBLE | WS_CAPTION | 
    WS_SYSMENU
CAPTION "xpilot:motd"
FONT 8, "MS Sans Serif"
BEGIN
    DEFPUSHBUTTON   "OK",IDOK,104,169,50,14
    EDITTEXT        IDC_EDIT1,13,8,232,158,ES_MULTILINE | ES_AUTOVSCROLL | 
                    ES_AUTOHSCROLL | ES_READONLY
END

IDD_TALKWINDOW DIALOG DISCARDABLE  0, 0, 414, 28
STYLE DS_MODALFRAME | WS_POPUP
FONT 8, "MS Sans Serif"
BEGIN
    EDITTEXT        IDC_EDIT1,7,7,400,14,ES_AUTOHSCROLL
END

IDD_EXPIRE_SOON DIALOG DISCARDABLE  0, 0, 193, 93
STYLE DS_MODALFRAME | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "Uh-oh"
FONT 12, "MS Sans Serif"
BEGIN
    DEFPUSHBUTTON   "OK",IDOK,71,72,50,14
    CTEXT           "This version of XPilotNT will expire at the end of this month.",
                    IDC_STATIC,11,11,171,8
    CTEXT           "Information about updates can be obtained at either:",
                    IDC_STATIC,16,22,151,8
    CTEXT           "http://www.buckosoft.com/xpilot/",IDC_STATIC,7,33,179,8
    CTEXT           "or",IDC_STATIC,92,46,8,8
    CTEXT           "news:rec.computer.games.xpilot",IDC_STATIC,7,61,179,8
    ICON            IDR_MAINFRAME,IDC_STATIC,54,46,20,20
    ICON            IDR_MAINFRAME,IDC_STATIC,127,46,20,20
END

IDD_SYSINFO DIALOG DISCARDABLE  0, 0, 258, 190
STYLE DS_MODALFRAME | DS_CENTER | WS_POPUP | WS_VISIBLE | WS_CAPTION | 
    WS_SYSMENU
CAPTION "System Info"
FONT 8, "MS Sans Serif"
BEGIN
    DEFPUSHBUTTON   "OK",IDOK,104,169,50,14
    EDITTEXT        IDC_EDIT1,13,58,232,108,ES_MULTILINE | ES_AUTOVSCROLL | 
                    ES_AUTOHSCROLL | ES_READONLY
    LTEXT           "Here is some information about your system that might be useful in understanding any problems you might encounter running XPilotNT on your computer.",
                    IDC_STATIC,7,8,244,19
    LTEXT           "Please include this information in any correspondance with dick@buckosoft.com about problems with XPilotNT",
                    IDC_STATIC,7,31,244,21
END

IDD_EXPIRED DIALOG DISCARDABLE  0, 0, 193, 93
STYLE DS_MODALFRAME | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "Uh-oh"
FONT 12, "MS Sans Serif"
BEGIN
    DEFPUSHBUTTON   "OK",IDOK,71,72,50,14
    CTEXT           "This version of XPilotNT has expired.",IDC_STATIC,7,11,
                    171,8
    CTEXT           "Information about updates can be obtained at either:",
                    IDC_STATIC,7,22,179,8
    CTEXT           "http://www.buckosoft.com/xpilot/",IDC_STATIC,7,33,179,8
    CTEXT           "or",IDC_STATIC,92,46,8,8
    CTEXT           "news:rec.computer.games.xpilot",IDC_STATIC,7,61,179,8
    ICON            IDR_MAINFRAME,IDC_STATIC,54,46,20,20
    ICON            IDR_MAINFRAME,IDC_STATIC,127,46,20,20
END


#ifndef _MAC
/////////////////////////////////////////////////////////////////////////////
//
// Version
//

VS_VERSION_INFO VERSIONINFO
 FILEVERSION 1,0,0,1
 PRODUCTVERSION 0,8,0,0
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x4L
 FILETYPE 0x1L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "000004b0"
        BEGIN
            VALUE "Comments", "ht\0"
            VALUE "CompanyName", "BuckoSoft\0"
            VALUE "FileDescription", "xpilot\0"
            VALUE "FileVersion", "0,8,0,0\0"
            VALUE "InternalName", "xpilot\0"
            VALUE "LegalCopyright", "Copyright © 1996-1997 BuckoSoft, Inc.\0"
            VALUE "OriginalFilename", "XPilotNT.exe\0"
            VALUE "ProductName", "XPilotNT\0"
            VALUE "ProductVersion", "0, 8, 0, 0\0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x0, 1200
    END
END

#endif    // !_MAC


/////////////////////////////////////////////////////////////////////////////
//
// DESIGNINFO
//

#ifdef APSTUDIO_INVOKED
GUIDELINES DESIGNINFO DISCARDABLE 
BEGIN
    IDD_ABOUTBOX, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 332
        TOPMARGIN, 7
        BOTTOMMARGIN, 233
    END

    IDD_MOTD, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 251
        TOPMARGIN, 8
        BOTTOMMARGIN, 183
    END

    IDD_TALKWINDOW, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 407
        TOPMARGIN, 7
        BOTTOMMARGIN, 21
    END

    IDD_EXPIRE_SOON, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 186
        TOPMARGIN, 7
        BOTTOMMARGIN, 86
    END

    IDD_SYSINFO, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 251
        TOPMARGIN, 7
        BOTTOMMARGIN, 183
    END

    IDD_EXPIRED, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 186
        TOPMARGIN, 7
        BOTTOMMARGIN, 86
    END
END
#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Menu
//

IDR_MAINFRAME MENU DISCARDABLE 
BEGIN
    MENUITEM "Quit",                        ID_FILE_CLOSE
    POPUP "&Configure"
    BEGIN
        MENUITEM "&Game",                       ID_CONFIGURE_GAME, GRAYED
        MENUITEM "&Colors",                     ID_COLORS, GRAYED
        MENUITEM "&Fonts",                      ID_CONFIGURE_FONTS, GRAYED
        MENUITEM SEPARATOR
        MENUITEM "Sys &Info",                   ID_CONFIGURE_SYSINFO
    END
    MENUITEM "Help",                        ID_HELP, GRAYED
    MENUITEM "About",                       ID_APP_ABOUT
END


/////////////////////////////////////////////////////////////////////////////
//
// String Table
//

STRINGTABLE DISCARDABLE 
BEGIN
    IDP_SOCKETS_INIT_FAILED "Windows sockets initialization failed."
END

STRINGTABLE PRELOAD DISCARDABLE 
BEGIN
    IDR_MAINFRAME           "XPilotNT\n\nXPilotNTt\n\n\nXpilot.Document\nXpilot Document"
END

STRINGTABLE PRELOAD DISCARDABLE 
BEGIN
    AFX_IDS_APP_TITLE       "XPilotNT"
    AFX_IDS_IDLEMESSAGE     "For Help, press F1"
    AFX_IDS_HELPMODEMESSAGE "Select an object on which to get Help"
END

STRINGTABLE DISCARDABLE 
BEGIN
    ID_INDICATOR_EXT        "EXT"
    ID_INDICATOR_CAPS       "CAP"
    ID_INDICATOR_NUM        "NUM"
    ID_INDICATOR_SCRL       "SCRL"
    ID_INDICATOR_OVR        "OVR"
    ID_INDICATOR_REC        "REC"
END

STRINGTABLE DISCARDABLE 
BEGIN
    ID_FILE_NEW             "Create a new document\nNew"
    ID_FILE_OPEN            "Open an existing document\nOpen"
    ID_FILE_CLOSE           "Close the active document\nClose"
    ID_FILE_SAVE            "Save the active document\nSave"
    ID_FILE_SAVE_AS         "Save the active document with a new name\nSave As"
END

STRINGTABLE DISCARDABLE 
BEGIN
    ID_APP_ABOUT            "Display program information, version number and copyright\nAbout"
    ID_APP_EXIT             "Quit the application; prompts to save documents\nExit"
    ID_HELP_INDEX           "Opens Help\nHelp Topics"
    ID_HELP_FINDER          "List Help topics\nHelp Topics"
    ID_HELP_USING           "Display instructions about how to use help\nHelp"
    ID_CONTEXT_HELP         "Display help for clicked on buttons, menus and windows\nHelp"
    ID_HELP                 "Display help for current task or command\nHelp"
END

STRINGTABLE DISCARDABLE 
BEGIN
    ID_FILE_MRU_FILE1       "Open this document"
    ID_FILE_MRU_FILE2       "Open this document"
    ID_FILE_MRU_FILE3       "Open this document"
    ID_FILE_MRU_FILE4       "Open this document"
    ID_FILE_MRU_FILE5       "Open this document"
    ID_FILE_MRU_FILE6       "Open this document"
    ID_FILE_MRU_FILE7       "Open this document"
    ID_FILE_MRU_FILE8       "Open this document"
    ID_FILE_MRU_FILE9       "Open this document"
    ID_FILE_MRU_FILE10      "Open this document"
    ID_FILE_MRU_FILE11      "Open this document"
    ID_FILE_MRU_FILE12      "Open this document"
    ID_FILE_MRU_FILE13      "Open this document"
    ID_FILE_MRU_FILE14      "Open this document"
    ID_FILE_MRU_FILE15      "Open this document"
    ID_FILE_MRU_FILE16      "Open this document"
END

STRINGTABLE DISCARDABLE 
BEGIN
    ID_NEXT_PANE            "Switch to the next window pane\nNext Pane"
    ID_PREV_PANE            "Switch back to the previous window pane\nPrevious Pane"
END

STRINGTABLE DISCARDABLE 
BEGIN
    ID_WINDOW_SPLIT         "Split the active window into panes\nSplit"
END

STRINGTABLE DISCARDABLE 
BEGIN
    ID_EDIT_CLEAR           "Erase the selection\nErase"
    ID_EDIT_CLEAR_ALL       "Erase everything\nErase All"
    ID_EDIT_COPY            "Copy the selection and put it on the Clipboard\nCopy"
    ID_EDIT_CUT             "Cut the selection and put it on the Clipboard\nCut"
    ID_EDIT_FIND            "Find the specified text\nFind"
    ID_EDIT_PASTE           "Insert Clipboard contents\nPaste"
    ID_EDIT_REPEAT          "Repeat the last action\nRepeat"
    ID_EDIT_REPLACE         "Replace specific text with different text\nReplace"
    ID_EDIT_SELECT_ALL      "Select the entire document\nSelect All"
    ID_EDIT_UNDO            "Undo the last action\nUndo"
    ID_EDIT_REDO            "Redo the previously undone action\nRedo"
END

STRINGTABLE DISCARDABLE 
BEGIN
    ID_VIEW_TOOLBAR         "Show or hide the toolbar\nToggle ToolBar"
    ID_VIEW_STATUS_BAR      "Show or hide the status bar\nToggle StatusBar"
END

STRINGTABLE DISCARDABLE 
BEGIN
    AFX_IDS_SCSIZE          "Change the window size"
    AFX_IDS_SCMOVE          "Change the window position"
    AFX_IDS_SCMINIMIZE      "Reduce the window to an icon"
    AFX_IDS_SCMAXIMIZE      "Enlarge the window to full size"
    AFX_IDS_SCNEXTWINDOW    "Switch to the next document window"
    AFX_IDS_SCPREVWINDOW    "Switch to the previous document window"
    AFX_IDS_SCCLOSE         "Close the active window and prompts to save the documents"
END

STRINGTABLE DISCARDABLE 
BEGIN
    AFX_IDS_SCRESTORE       "Restore the window to normal size"
    AFX_IDS_SCTASKLIST      "Activate Task List"
END

#endif    // English (U.S.) resources
/////////////////////////////////////////////////////////////////////////////



#ifndef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//
#define _AFX_NO_SPLITTER_RESOURCES
#define _AFX_NO_OLE_RESOURCES
#define _AFX_NO_TRACKER_RESOURCES
#define _AFX_NO_PROPERTY_RESOURCES

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENU)
#ifdef _WIN32
LANGUAGE 9, 1
#pragma code_page(1252)
#endif
#include "res\xpilot.rc2"  // non-Microsoft Visual C++ edited resources
#include "afxres.rc"         // Standard components
#endif
/////////////////////////////////////////////////////////////////////////////
#endif    // not APSTUDIO_INVOKED

