/* 
 * XPilot NG, a multiplayer space war game.
 *
 * Copyright (C) 2005 Kristian Söderblom <kps@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef XPMAP_H
#define XPMAP_H

#define XPMAP_SPACE			' '
#define XPMAP_SPACE_ALT			'.'
#define XPMAP_BASE			'_'
#define XPMAP_BASE_TEAM_0		'0'
#define XPMAP_BASE_TEAM_1		'1'
#define XPMAP_BASE_TEAM_2		'2'
#define XPMAP_BASE_TEAM_3		'3'
#define XPMAP_BASE_TEAM_4		'4'
#define XPMAP_BASE_TEAM_5		'5'
#define XPMAP_BASE_TEAM_6		'6'
#define XPMAP_BASE_TEAM_7		'7'
#define XPMAP_BASE_TEAM_8		'8'
#define XPMAP_BASE_TEAM_9		'9'
#define XPMAP_BASE_ATTRACTOR		'$'
#define XPMAP_FILLED			'x'
#define XPMAP_REC_LU			's'
#define XPMAP_REC_LD			'w'
#define XPMAP_REC_RU			'a'
#define XPMAP_REC_RD			'q'
#define XPMAP_FUEL			'#'
#define XPMAP_CANNON_UP			'r'
#define XPMAP_CANNON_LEFT		'd'
#define XPMAP_CANNON_RIGHT		'f'
#define XPMAP_CANNON_DOWN		'c'
#define XPMAP_POS_GRAV			'+'
#define XPMAP_NEG_GRAV			'-'
#define XPMAP_CWISE_GRAV		'>'
#define XPMAP_ACWISE_GRAV		'<'
#define XPMAP_UP_GRAV			'i'
#define XPMAP_DOWN_GRAV			'm'
#define XPMAP_RIGHT_GRAV		'k'
#define XPMAP_LEFT_GRAV			'j'
#define XPMAP_WORMHOLE_NORMAL		'@'
#define XPMAP_WORMHOLE_IN		'('
#define XPMAP_WORMHOLE_OUT		')'
#define XPMAP_TREASURE			'*'
#define XPMAP_EMPTY_TREASURE		'^'
#define XPMAP_TARGET			'!'
#define XPMAP_ITEM_CONCENTRATOR		'%'
#define XPMAP_ASTEROID_CONCENTRATOR	'&'
#define XPMAP_DECOR_FILLED		'b'
#define XPMAP_DECOR_LU			'h'
#define XPMAP_DECOR_LD			'y'
#define XPMAP_DECOR_RU			'g'
#define XPMAP_DECOR_RD			't'
#define XPMAP_FRICTION_AREA		'z'
#define XPMAP_CHECK_0			'A'
#define XPMAP_CHECK_1			'B'
#define XPMAP_CHECK_2			'C'
#define XPMAP_CHECK_3			'D'
#define XPMAP_CHECK_4			'E'
#define XPMAP_CHECK_5			'F'
#define XPMAP_CHECK_6			'G'
#define XPMAP_CHECK_7			'H'
#define XPMAP_CHECK_8			'I'
#define XPMAP_CHECK_9			'J'
#define XPMAP_CHECK_10			'K'
#define XPMAP_CHECK_11			'L'
#define XPMAP_CHECK_12			'M'
#define XPMAP_CHECK_13			'N'
#define XPMAP_CHECK_14			'O'
#define XPMAP_CHECK_15			'P'
#define XPMAP_CHECK_16			'Q'
#define XPMAP_CHECK_17			'R'
#define XPMAP_CHECK_18			'S'
#define XPMAP_CHECK_19			'T'
#define XPMAP_CHECK_20			'U'
#define XPMAP_CHECK_21			'V'
#define XPMAP_CHECK_22			'W'
#define XPMAP_CHECK_23			'X'
#define XPMAP_CHECK_24			'Y'
#define XPMAP_CHECK_25			'Z'

#endif
