// credits.inc.h
// Generated by ./creditsUpdate.pl

credits = "";
credits += "#r001This is a very abbreviated credit list.  We couldn't possibly enumerate all";
credits += "#r001the people who have been involved in XPilot, especially since we have no such";
credits += "#r001list at hand. :)  If you find someone (yourself!?) missing from this list";
credits += "#r001then please don't hesitate to complain to the authors at xpilot@xpilot.org.";
credits += "#r002xpilot.org is:";
credits += "#r001#c005Bert Gijsbers#c025xpilot@xpilot.org";
credits += "#r001#c005Dick Balaska";
credits += "#r001#c005Kimiko Koopman";
credits += "#r001#c005Mark Boyns#c025DNS services";
credits += "#r001#c005Scott Adkins#c025www, ftp, and meta2 services";
credits += "#r001#c005Dick Balaska#c025mail services";
credits += "#r001#c005uni-Tromso, Norway#c025meta services";
credits += "#r002Release 4.5 credits:";
credits += "#r001#c005Jarno van der Kolk#c025Bugfix, Windows recording,";
credits += "#r001#c025draw asteroid concentrators.";
credits += "#r001#c005Paul Alexander Drees#c025Asteroid Concentrators.";
credits += "#r001#c005Kimiko Koopman#c025Ball racing, kinetic energy in";
credits += "#r001#c025shot hits, scores as floats,";
credits += "#r001#c025alliances, cannon defense.";
credits += "#r001#c005Austin Donnelly#c025Auto-kick of paused players.";
credits += "#r001#c005Paul Alexander Drees#c025Bugfixes, sensitive mines,";
credits += "#r001#c025asteroid concentrators, lag meter.";
credits += "#r001#c005Thorsten Kroeger#c025Messages to standard output.";
credits += "#r001#c025server security fix.";
credits += "#r002Release 4.4 credits:";
credits += "#r001#c005Paul Alexander Drees#c025Lots of small new features + bugfixes.";
credits += "#r001#c005Guido Koopman#c025Balls in non-team-mode, asteroids.";
credits += "#r001#c005Ben Jackson#c025Bugfix.";
credits += "#r001#c005Paul Forman#c025Disallow stationary mines.";
credits += "#r001#c005Jarrod Miller#c025Mapeditor updates";
credits += "#r001#c005Austin Donnelly#c025OSF/1 fixes, ball string & collisions.";
credits += "#r001#c005Ben Armstrong#c025Documentation, bugfixes.";
credits += "#r001#c005Joe Groff#c025Multiple keys per mouse button.";
credits += "#r001#c005Jarno van der Kolk#c025More server tuning, Windows XPreplay.";
credits += "#r001#c025Windows NSIS Install script.";
credits += "#r001#c005Bob Glickstein#c025Flak.";
credits += "#r001#c005Juha Lindström#c025radar on HUD.Release 4.3 credits:";
credits += "#r002#c005Claus Leth Gregersen#c025Several patches, graphics look.";
credits += "#r001#c005Marcus Sundberg#c025Radar color fix + localnet fix.";
credits += "#r001#c005Jarrod Miller#c025Fix base orientation, update mapeditor.";
credits += "#r001#c005Peder Herborg#c025Fixes for sliding radar under Windows.";
credits += "#r001#c005Ben Jackson#c025Several fixes + ping button.";
credits += "#r001#c005Jarno van der Kolk#c025Windows sliding radar performance fix.";
credits += "#r001#c005Juha Lindström#c025Windows fixes for new socklib.";
credits += "#r001#c005Ben Armstrong#c025UDP port range + ideas + documentation.";
credits += "#r002Release 4.2 credits:";
credits += "#r001#c005Ben Jackson#c025Numerous patches.";
credits += "#r001#c005Uoti Urpala#c025Numerous patches.";
credits += "#r001#c005Jonny Svärling,#c025Talk macros.";
credits += "#r001#c005Adam Lorentzon";
credits += "#r001#c005Teemu Elias Hirsimäki#c025xp-replay fix.";
credits += "#r001#c005Guido Koopman#c025Armor item.";
credits += "#r001#c005Sven Mascheck#c025Joined the maintenance team.";
credits += "#r002Release 4.1 credits:";
credits += "#r001#c005Guido Koopman#c025Several patches + 1 new item.";
credits += "#r001#c005Sven Mascheck#c025Erase work + fixes & testing.";
credits += "#r001#c005Rakk & Murx#c025Analytical Collision Detection.";
credits += "#r002Release 4.0 credits:";
credits += "#r001#c005Guido Koopman#c025Several patches + 3 new items";
credits += "#r001#c025+ much improved robots.";
credits += "#r001#c005Rodney R. Korte#c025Port to OS/2 + fixes.";
credits += "#r001#c005Sven Mascheck#c025Bugfixes + snooping.";
credits += "#r001#c005Mark Lambert#c025Memory leak fix.";
credits += "#r001#c005Espen Skoglund#c025Ship wreckages.";
credits += "#r002Release 3.7 credits:";
credits += "#r001#c005Dick Balaska#c025Port to Windows NT & 95.";
credits += "#r001#c005Martin Sandiford#c025Windows port debugging + fixes.";
credits += "#r001#c005E. Hugh Woods III#c025Windows installation script.";
credits += "#r001#c005Mark Boyns#c025Ideas and discussions.";
credits += "#r001#c005Jens Koerber#c025VMS patches.";
credits += "#r001#c005Ben Aveling#c025Several speed improvements + meaner robots.";
credits += "#r001#c005Santeri Paavolainen#c025X11 Double Buffering extension.";
credits += "#r001#c005David Leonard#c025msub fix for BSD.";
credits += "#r001#c005Michael Driscoll#c025Diff for README.MAPS2";
credits += "#r001#c005Guido Koopman#c025Several new patches.";
credits += "#r002Release 3.6 credits:";
credits += "#r001#c005Tim Auckland#c025Gravity currents";
credits += "#r001#c005Mark Boyns#c025Several patches and comments";
credits += "#r001#c005Scott Manley";
credits += "#r001#c005Ben Jackson";
credits += "#r001#c005Andrew Mobbs#c025itemConcentratorProb";
credits += "#r001#c005Ian Jackson#c025itemConcentratorProb";
credits += "#r002Release 3.5 credits:";
credits += "#r001#c005Steve Payne#c025Ship checking, Linux plock + option";
credits += "#r001#c005Claus Leth Gregersen#c025Item limits, hud ship, textured balls";
credits += "#r001#c005Aleksi Suhonen#c025X multibuffer";
credits += "#r001#c005Jens Koerber#c025VMS changes";
credits += "#r002Release 3.4 credits:";
credits += "#r001#c005Mark Boyns#c025Map decorations + other things.";
credits += "#r001#c005Michael Gruber#c025Left & right rear guns.";
credits += "#r001#c005Jens Koerber#c025Changes for VMS.";
credits += "#r001#c005Bretton Wade#c025Ball movement research + code.";
credits += "#r001#c005Christian Montanari#c025Changes to HP audio.";
credits += "#r001#c005Tom Hansen#c025Drop items.";
credits += "#r002Release 3.3 credits:";
credits += "#r001#c005Steven Singer#c025Record & playback, filled walls.";
credits += "#r001#c005Christian Engstler#c025Bug reports & Sun guinea pig.";
credits += "#r002Release 3.2 credits:";
credits += "#r001#c005Harvey Thompson#c025Bugfixes and new code.";
credits += "#r001#c005Scott W. Adkins#c025Emergency Shield.";
credits += "#r001#c005John E. Norlin#c025Robots ship shapes.";
credits += "#r001#c005Patrick Kenny#c025GOD log, bugfix, Blood's Music.";
credits += "#r001#c005Pete Ford#c025New ship shape format.";
credits += "#r001#c005Jyke Jokinen#c025xpilot host alias.";
credits += "#r001#c005Russ Glaeser#c025Better scoring system.";
credits += "#r001#c005Steven Singer#c025Mouse control, race mode.";
credits += "#r001#c005Nicholas Vargish#c025rogues options & persistence ;-)";
credits += "#r001#c005Joshua Yelon#c025Better explosion system, shove patch.";
credits += "#r001#c005Eckard Kopatzki#c025Joystick support for Linux.";
credits += "#r001#c005Tom De Pauw &#c025AF audio support.";
credits += "#r001#c005Lance Berc";
credits += "#r001#c005Ben Jackson#c025Bug fixes.";
credits += "#r001#c005Steve Marsden#c025User controllable mine detonation.";
credits += "#r002Release 3.1 credits:";
credits += "#r001#c005Harvey Thompson#c025Lots of first class wonderful new";
credits += "#r001#c025features, ideas and cleanups.  Thanks!";
credits += "#r001#c005Mark Boyns#c025Sound.";
credits += "#r001#c005Stig Sæther Bakken#c025Imakefile, testing & feedback.";
credits += "#r001#c005Scott W. Adkins#c025Ship shapes.";
credits += "#r001#c005Tony Plate#c025Missile length and names.";
credits += "#r001#c005Steven Singer#c025Sun tester & knowledge provider. :-)";
credits += "#r002Release 3.0 credits:";
credits += "#r001#c005Maurice Abraham#c025Robots, more flexible bouncing etc.";
credits += "#r001#c005Mark Boyns &#c025RPlay - original sound support.";
credits += "#r001#c005David J. Bianco";
credits += "#r001#c005Greg Renda#c025Sound scheme client/serverized";
credits += "#r001#c025and added NCD sound support.";
credits += "#r001#c005Dave Lemke";
credits += "#r001#c005Curt Hjorring#c025VMS port.";
credits += "#r001#c005Andrew Scherpbier#c025Random sound patch.  Xpmtp.";
credits += "#r001#c005Steven Singer#c025Sun tester & knowledge provider. :-)";
credits += "#r001#c005Andy Skinner";
credits += "#r001#c005Tero Kivinen#c025Outline world.";
credits += "#r001#c005David Chatterton &#c025Ultrix and SGI.";
credits += "#r001#c005Johan Danielsson";
credits += "#r001#c005Fred Hucht &#c025Several (big) patches";
credits += "#r001#c005Michael Staats";
credits += "#r001#c005Maurice Abraham";
credits += "#r001#c005Makoto Matsushita";
credits += "#r001#c005Stig Sæther Bakken#c025patches & feedback.";
credits += "#r001#c005& all the beta testers";
credits += "#r002Version 2.0 credits:    (Never released)";
credits += "#r002#c005Bert Gijsbers#c025Too much to mention really.";
credits += "#r001#c005Robert Blum &#c025Lots of features.";
credits += "#r001#c005more from Aachen";
credits += "#r001#c005Andy Skinner#c025Target code and maps.";
credits += "#r001#c005Paul Gardner#c025Random map generator + optimisations.";
credits += "#r001#c005Greg Renda";
credits += "#r001#c005Randall Severly";
credits += "#r001#c005Evan Miller";
credits += "#r001#c005Eric Flink";
credits += "#r001#c005Karthik P. Sheka";
credits += "#r001#c005Stig Sæther Bakken";
credits += "#r001#c005Jim Hickey";
credits += "#r001#c005Harvey Thompson";
credits += "#r001#c005Ted Lemon";
credits += "#r002Version 1.2 credits:";
credits += "#r001#c005Christos Zoulas#c025Monochrome fixes";
credits += "#r001#c005Abraham Maurice#c025Robots, timing.";
credits += "#r001#c005Greg Renda#c025Rebinding of keys, wormholes etc.";
credits += "#r001#c005Dave Lemke";
credits += "#r001#c005Keith Cantrell";
credits += "#r001#c005Chris Marshall";
credits += "#r001#c005Sigmund Austigard";
credits += "#r001#c005Martin Rutishauser";
credits += "#r001#c005Frank Tore Johansen";
credits += "#r001#c005Hermann Dunkel";
credits += "#r002Pre version 1.2 credits:";
credits += "#r001#c005Arne Helme#c025Socklib";
credits += "#r001#c005Tom Grydeland#c025Maps, fonts and ideas";
credits += "#r002#c005Lots of, LOTS OF beta";
credits += "#r001#c005testers at our site.";
credits += "#r000";
lineCount = 193;
