/* 
 * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-2001 by
 *
 *      Bjørn Stabell        <bjoern@xpilot.org>
 *      Ken Ronny Schouten   <ken@xpilot.org>
 *      Bert Gijsbers        <bert@xpilot.org>
 *      Dick Balaska         <dick@xpilot.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/***************************************************************************\
*  Splash.h - The Splash Panel for XPilotNT									*
*																			*
*  This file is the standard splash component from MSDEV enhanced to allow	*
*  displaying progress messages in the lower left corner.					*
*																			*
*  							*
\***************************************************************************/
// CG: This file was added by the Splash Screen component.

#ifndef _SPLASH_SCRN_
#define _SPLASH_SCRN_

// Splash.h : header file
//

// Splash uses Predident Regular 12 for the version font.
/////////////////////////////////////////////////////////////////////////////
//   Splash Screen class

class CSplashWnd:public CWnd {
// Construction
  protected:
    CSplashWnd();

// Attributes:
  public:
    CBitmap m_bitmap;

// Operations
  public:
    static void EnableSplashScreen(BOOL bEnable = TRUE);
    static void ShowSplashScreen(CWnd * pParentWnd = NULL);
    static BOOL PreTranslateAppMessage(MSG * pMsg);

    static void ShowMessage(const CString & msg);

// Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CSplashWnd)
    //}}AFX_VIRTUAL

// Implementation
  public:
    ~CSplashWnd();
    virtual void PostNcDestroy();


  protected:
     BOOL Create(CWnd * pParentWnd = NULL);
    void HideSplashScreen();
    static BOOL c_bShowSplashWnd;
    static CSplashWnd *c_pSplashWnd;

// Generated message map functions
  protected:
    //{{AFX_MSG(CSplashWnd)
     afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
    afx_msg void OnPaint();
    afx_msg void OnTimer(UINT nIDEvent);
    //}}AFX_MSG
     DECLARE_MESSAGE_MAP()
};


#endif
